package sun.print;

import java.util.Arrays;
import java.util.Comparator;
import java.security.PrivilegedAction;
import javax.print.MultiDocPrintService;
import java.util.ArrayList;
import javax.print.attribute.standard.PrinterName;
import javax.print.attribute.PrintRequestAttribute;
import javax.print.attribute.HashPrintServiceAttributeSet;
import javax.print.attribute.HashPrintRequestAttributeSet;
import javax.print.attribute.AttributeSet;
import javax.print.DocFlavor;
import javax.print.attribute.PrintServiceAttribute;
import javax.print.attribute.Attribute;
import javax.print.attribute.PrintServiceAttributeSet;
import java.security.AccessController;
import sun.security.action.GetPropertyAction;
import javax.print.PrintService;
import javax.print.PrintServiceLookup;

public class Win32PrintServiceLookup extends PrintServiceLookup
{
    private String defaultPrinter;
    private PrintService defaultPrintService;
    private String[] printers;
    private PrintService[] printServices;
    private static final int DEFAULT_REFRESH_TIME = 240;
    private static final int MINIMUM_REFRESH_TIME = 120;
    private static final boolean pollServices;
    private static final int refreshTime;
    private static Win32PrintServiceLookup win32PrintLUS;
    
    private static int getRefreshTime(String s) {
        try {
            int int1 = Integer.parseInt(s);
            return (int1 < 120) ? 120 : int1;
        }
        catch (NumberFormatException ex) {
            return 240;
        }
    }
    
    public static Win32PrintServiceLookup getWin32PrintLUS() {
        if (Win32PrintServiceLookup.win32PrintLUS == null) {
            PrintServiceLookup.lookupDefaultPrintService();
        }
        return Win32PrintServiceLookup.win32PrintLUS;
    }
    
    public Win32PrintServiceLookup() {
        if (Win32PrintServiceLookup.win32PrintLUS == null) {
            Win32PrintServiceLookup.win32PrintLUS = this;
            String s = (String)AccessController.doPrivileged(new GetPropertyAction("os.name"));
            if (s != null && s.startsWith("Windows 98")) {
                return;
            }
            Thread thread = new Thread(null, new Win32PrintServiceLookup.PrinterChangeListener(), "PrinterListener", 0L);
            thread.setDaemon(true);
            thread.start();
            if (Win32PrintServiceLookup.pollServices) {
                Thread thread2 = new Thread(null, new Win32PrintServiceLookup.RemotePrinterChangeListener(), "RemotePrinterListener", 0L);
                thread2.setDaemon(true);
                thread2.start();
            }
        }
    }
    
    @Override
    public synchronized PrintService[] getPrintServices() {
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkPrintJobAccess();
        }
        if (this.printServices == null) {
            this.refreshServices();
        }
        return this.printServices;
    }
    
    private synchronized void refreshServices() {
        this.printers = this.getAllPrinterNames();
        if (this.printers == null) {
            this.printServices = new PrintService[0];
            return;
        }
        PrintService[] printServices = new PrintService[this.printers.length];
        PrintService defaultPrintService = this.getDefaultPrintService();
        for (int i = 0; i < this.printers.length; ++i) {
            if (defaultPrintService != null && this.printers[i].equals(defaultPrintService.getName())) {
                printServices[i] = defaultPrintService;
            }
            else if (this.printServices == null) {
                printServices[i] = new Win32PrintService(this.printers[i]);
            }
            else {
                int j = 0;
                while (j < this.printServices.length) {
                    if (this.printServices[j] != null && this.printers[i].equals(this.printServices[j].getName())) {
                        printServices[i] = this.printServices[j];
                        this.printServices[j] = null;
                        break;
                    }
                    ++j;
                }
                if (j == this.printServices.length) {
                    printServices[i] = new Win32PrintService(this.printers[i]);
                }
            }
        }
        if (this.printServices != null) {
            for (int k = 0; k < this.printServices.length; ++k) {
                if (this.printServices[k] instanceof Win32PrintService && !this.printServices[k].equals(this.defaultPrintService)) {
                    ((Win32PrintService)this.printServices[k]).invalidateService();
                }
            }
        }
        this.printServices = printServices;
        return;
    }
    
    public synchronized PrintService getPrintServiceByName(String anObject) {
        if (anObject == null || anObject.equals("")) {
            return null;
        }
        PrintService[] printServices = this.getPrintServices();
        int i = 0;
        while (i < printServices.length) {
            if (printServices[i].getName().equals(anObject)) {
                return printServices[i];
            }
            ++i;
        }
        return null;
    }
    
    boolean matchingService(PrintService printService, PrintServiceAttributeSet set) {
        if (set != null) {
            Attribute[] array = set.toArray();
            int i = 0;
            while (i < array.length) {
                PrintServiceAttribute attribute = printService.getAttribute(array[i].getCategory());
                if (attribute == null || !attribute.equals(array[i])) {
                    return false;
                }
                ++i;
            }
        }
        return true;
    }
    
    @Override
    public PrintService[] getPrintServices(DocFlavor docFlavor, AttributeSet set) {
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkPrintJobAccess();
        }
        HashPrintRequestAttributeSet set2 = null;
        HashPrintServiceAttributeSet set3 = null;
        if (set != null && !set.isEmpty()) {
            set2 = new HashPrintRequestAttributeSet();
            set3 = new HashPrintServiceAttributeSet();
            Attribute[] array = set.toArray();
            for (int i = 0; i < array.length; ++i) {
                if ((array[i] instanceof PrintRequestAttribute)) {
                    set2.add(array[i]);
                }
                else if ((array[i] instanceof PrintServiceAttribute)) {
                    set3.add(array[i]);
                }
            }
        }
        PrintService[] printServices;
        if (set3 != null && set3.get(PrinterName.class) != null) {
            PrintService printServiceByName = this.getPrintServiceByName(((PrinterName)set3.get(PrinterName.class)).getValue());
            if (printServiceByName == null || !this.matchingService(printServiceByName, set3)) {
                printServices = new PrintService[0];
            }
            else {
                printServices = new PrintService[] { printServiceByName };
            }
        }
        else {
            printServices = this.getPrintServices();
        }
        if (printServices.length == 0) {
            return printServices;
        }
        ArrayList o = new ArrayList();
        for (int j = 0; j < printServices.length; ++j) {
            try {
                if (printServices[j].getUnsupportedAttributes(docFlavor, set2) == null) {
                    o.add(printServices[j]);
                }
            }
            catch (IllegalArgumentException ex) {}
        }
        return (PrintService[])(PrintService[])o.toArray(new PrintService[o.size()]);
    }
    
    @Override
    public MultiDocPrintService[] getMultiDocPrintServices(DocFlavor[] array, AttributeSet set) {
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkPrintJobAccess();
        }
        return new MultiDocPrintService[0];
    }
    
    @Override
    public synchronized PrintService getDefaultPrintService() {
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkPrintJobAccess();
        }
        this.defaultPrinter = this.getDefaultPrinterName();
        if (this.defaultPrinter == null) {
            return null;
        }
        if (this.defaultPrintService != null && this.defaultPrintService.getName().equals(this.defaultPrinter)) {
            return this.defaultPrintService;
        }
        this.defaultPrintService = null;
        if (this.printServices != null) {
            int i = 0;
            while (i < this.printServices.length) {
                if (this.defaultPrinter.equals(this.printServices[i].getName())) {
                    this.defaultPrintService = this.printServices[i];
                    break;
                }
                ++i;
            }
        }
        if (this.defaultPrintService == null) {
            this.defaultPrintService = new Win32PrintService(this.defaultPrinter);
        }
        return this.defaultPrintService;
    }
    
    private native String getDefaultPrinterName();
    
    private native String[] getAllPrinterNames();
    
    private native long notifyFirstPrinterChange(String p0);
    
    private native void notifyClosePrinterChange(long p0);
    
    private native int notifyPrinterChange(long p0);
    
    private native String[] getRemotePrintersNames();
    
    static long access$000(Win32PrintServiceLookup win32PrintServiceLookup, String s) {
        return win32PrintServiceLookup.notifyFirstPrinterChange(s);
    }
    
    static int access$100(Win32PrintServiceLookup win32PrintServiceLookup, long n) {
        return win32PrintServiceLookup.notifyPrinterChange(n);
    }
    
    static void access$200(Win32PrintServiceLookup win32PrintServiceLookup) {
        win32PrintServiceLookup.refreshServices();
    }
    
    static void access$300(Win32PrintServiceLookup win32PrintServiceLookup, long n) {
        win32PrintServiceLookup.notifyClosePrinterChange(n);
    }
    
    static String[] access$400(Win32PrintServiceLookup win32PrintServiceLookup) {
        return win32PrintServiceLookup.getRemotePrintersNames();
    }
    
    static {
        pollServices = !"false".equalsIgnoreCase((String)AccessController.doPrivileged(new GetPropertyAction("sun.java2d.print.polling")));
        String s = (String)AccessController.doPrivileged(new GetPropertyAction("sun.java2d.print.minRefreshTime"));
        refreshTime = ((s != null) ? getRefreshTime(s) : 240);
        AccessController.doPrivileged(new PrivilegedAction<Void>() {
            @Override
            public Void run() {
                System.loadLibrary("awt");
                return null;
            }
        });
    }
    
    class PrinterChangeListener implements Runnable
    {
        long chgObj;
        
        PrinterChangeListener() {
            this.chgObj = Win32PrintServiceLookup.access$000(Win32PrintServiceLookup.this, null);
        }
        
        @Override
        public void run() {
            if (this.chgObj != -1L) {
                while (Win32PrintServiceLookup.access$100(Win32PrintServiceLookup.this, this.chgObj) != 0) {
                    try {
                        Win32PrintServiceLookup.access$200(Win32PrintServiceLookup.this);
                    }
                    catch (SecurityException ex) {
                        return;
                    }
                }
                Win32PrintServiceLookup.access$300(Win32PrintServiceLookup.this, this.chgObj);
            }
        }
    }
    
    class RemotePrinterChangeListener implements Comparator<String>, Runnable
    {
        @Override
        public int compare(String s, String anotherString) {
            return (s == null) ? ((anotherString == null) ? 0 : 1) : ((anotherString == null) ? -1 : s.compareTo(anotherString));
        }
        
        @Override
        public void run() {
            String[] access$400 = Win32PrintServiceLookup.access$400(Win32PrintServiceLookup.this);
            if (access$400 != null) {
                Arrays.sort(access$400, this);
            }
            while (true) {
                try {
                    Thread.sleep((long)(Win32PrintServiceLookup.refreshTime * 1000));
                }
                catch (InterruptedException ex) {
                    break;
                }
                String[] access$401 = Win32PrintServiceLookup.access$400(Win32PrintServiceLookup.this);
                if (access$401 != null) {
                    Arrays.sort(access$401, this);
                }
                if (!Arrays.equals(access$400, access$401)) {
                    Win32PrintServiceLookup.access$200(Win32PrintServiceLookup.this);
                    access$400 = access$401;
                }
                else {
                    continue;
                }
            }
        }
    }
}
